@@ -10,10 +10,12 @@ use crate::lexer::CommentType;
 pub struct Loc(pub usize, pub usize, pub usize);
 
 impl Loc {
+    #[must_use]
     pub fn begin(&self) -> Self {
         Loc(self.0, self.1, self.1)
     }
 
+    #[must_use]
     pub fn end(&self) -> Self {
         Loc(self.0, self.2, self.2)
     }
@@ -25,6 +27,12 @@ pub struct Identifier {
     pub name: String,
 }
 
+#[derive(Debug, PartialEq, Clone)]
+pub enum Comment {
+    Line(Loc, String),
+    Block(Loc, String),
+}
+
 #[derive(Debug, PartialEq, Clone)]
 pub enum DocComment {
     Line { comment: SingleDocComment },
@@ -53,11 +61,12 @@ pub struct SourceUnit(pub Vec<SourceUnitPart>);
 #[derive(Debug, PartialEq)]
 pub enum SourceUnitPart {
     ContractDefinition(Box<ContractDefinition>),
-    PragmaDirective(Vec<DocComment>, Identifier, StringLiteral),
+    PragmaDirective(Loc, Vec<DocComment>, Identifier, StringLiteral),
     ImportDirective(Vec<DocComment>, Import),
     EnumDefinition(Box<EnumDefinition>),
     StructDefinition(Box<StructDefinition>),
     EventDefinition(Box<EventDefinition>),
+    ErrorDefinition(Box<ErrorDefinition>),
     FunctionDefinition(Box<FunctionDefinition>),
     VariableDefinition(Box<VariableDefinition>),
     StraySemicolon(Loc),
@@ -65,9 +74,9 @@ pub enum SourceUnitPart {
 
 #[derive(Debug, PartialEq)]
 pub enum Import {
-    Plain(StringLiteral),
-    GlobalSymbol(StringLiteral, Identifier),
-    Rename(StringLiteral, Vec<(Identifier, Option<Identifier>)>),
+    Plain(StringLiteral, Loc),
+    GlobalSymbol(StringLiteral, Identifier, Loc),
+    Rename(StringLiteral, Vec<(Identifier, Option<Identifier>)>, Loc),
 }
 
 #[derive(Debug, PartialEq, Clone)]
@@ -140,6 +149,7 @@ pub enum ContractPart {
     StructDefinition(Box<StructDefinition>),
     EventDefinition(Box<EventDefinition>),
     EnumDefinition(Box<EnumDefinition>),
+    ErrorDefinition(Box<ErrorDefinition>),
     VariableDefinition(Box<VariableDefinition>),
     FunctionDefinition(Box<FunctionDefinition>),
     StraySemicolon(Loc),
@@ -206,6 +216,21 @@ pub struct EventDefinition {
     pub anonymous: bool,
 }
 
+#[derive(Debug, PartialEq)]
+pub struct ErrorParameter {
+    pub ty: Expression,
+    pub loc: Loc,
+    pub name: Option<Identifier>,
+}
+
+#[derive(Debug, PartialEq)]
+pub struct ErrorDefinition {
+    pub doc: Vec<DocComment>,
+    pub loc: Loc,
+    pub name: Identifier,
+    pub fields: Vec<ErrorParameter>,
+}
+
 #[derive(Debug, PartialEq)]
 pub struct EnumDefinition {
     pub doc: Vec<DocComment>,
@@ -270,6 +295,12 @@ pub enum Expression {
     PostDecrement(Loc, Box<Expression>),
     New(Loc, Box<Expression>),
     ArraySubscript(Loc, Box<Expression>, Option<Box<Expression>>),
+    ArraySlice(
+        Loc,
+        Box<Expression>,
+        Option<Box<Expression>>,
+        Option<Box<Expression>>,
+    ),
     MemberAccess(Loc, Box<Expression>, Identifier),
     FunctionCall(Loc, Box<Expression>, Vec<Expression>),
     FunctionCallBlock(Loc, Box<Expression>, Box<Statement>),
@@ -333,11 +364,12 @@ impl Expression {
             Expression::PostIncrement(loc, _)
             | Expression::PostDecrement(loc, _)
             | Expression::New(loc, _)
-            | Expression::ArraySubscript(loc, _, _)
-            | Expression::MemberAccess(loc, _, _)
-            | Expression::FunctionCall(loc, _, _)
-            | Expression::FunctionCallBlock(loc, _, _)
-            | Expression::NamedFunctionCall(loc, _, _)
+            | Expression::ArraySubscript(loc, ..)
+            | Expression::ArraySlice(loc, ..)
+            | Expression::MemberAccess(loc, ..)
+            | Expression::FunctionCall(loc, ..)
+            | Expression::FunctionCallBlock(loc, ..)
+            | Expression::NamedFunctionCall(loc, ..)
             | Expression::Not(loc, _)
             | Expression::Complement(loc, _)
             | Expression::Delete(loc, _)
@@ -345,37 +377,37 @@ impl Expression {
             | Expression::PreDecrement(loc, _)
             | Expression::UnaryPlus(loc, _)
             | Expression::UnaryMinus(loc, _)
-            | Expression::Power(loc, _, _)
-            | Expression::Multiply(loc, _, _)
-            | Expression::Divide(loc, _, _)
-            | Expression::Modulo(loc, _, _)
-            | Expression::Add(loc, _, _)
-            | Expression::Subtract(loc, _, _)
-            | Expression::ShiftLeft(loc, _, _)
-            | Expression::ShiftRight(loc, _, _)
-            | Expression::BitwiseAnd(loc, _, _)
-            | Expression::BitwiseXor(loc, _, _)
-            | Expression::BitwiseOr(loc, _, _)
-            | Expression::Less(loc, _, _)
-            | Expression::More(loc, _, _)
-            | Expression::LessEqual(loc, _, _)
-            | Expression::MoreEqual(loc, _, _)
-            | Expression::Equal(loc, _, _)
-            | Expression::NotEqual(loc, _, _)
-            | Expression::And(loc, _, _)
-            | Expression::Or(loc, _, _)
-            | Expression::Ternary(loc, _, _, _)
-            | Expression::Assign(loc, _, _)
-            | Expression::AssignOr(loc, _, _)
-            | Expression::AssignAnd(loc, _, _)
-            | Expression::AssignXor(loc, _, _)
-            | Expression::AssignShiftLeft(loc, _, _)
-            | Expression::AssignShiftRight(loc, _, _)
-            | Expression::AssignAdd(loc, _, _)
-            | Expression::AssignSubtract(loc, _, _)
-            | Expression::AssignMultiply(loc, _, _)
-            | Expression::AssignDivide(loc, _, _)
-            | Expression::AssignModulo(loc, _, _)
+            | Expression::Power(loc, ..)
+            | Expression::Multiply(loc, ..)
+            | Expression::Divide(loc, ..)
+            | Expression::Modulo(loc, ..)
+            | Expression::Add(loc, ..)
+            | Expression::Subtract(loc, ..)
+            | Expression::ShiftLeft(loc, ..)
+            | Expression::ShiftRight(loc, ..)
+            | Expression::BitwiseAnd(loc, ..)
+            | Expression::BitwiseXor(loc, ..)
+            | Expression::BitwiseOr(loc, ..)
+            | Expression::Less(loc, ..)
+            | Expression::More(loc, ..)
+            | Expression::LessEqual(loc, ..)
+            | Expression::MoreEqual(loc, ..)
+            | Expression::Equal(loc, ..)
+            | Expression::NotEqual(loc, ..)
+            | Expression::And(loc, ..)
+            | Expression::Or(loc, ..)
+            | Expression::Ternary(loc, ..)
+            | Expression::Assign(loc, ..)
+            | Expression::AssignOr(loc, ..)
+            | Expression::AssignAnd(loc, ..)
+            | Expression::AssignXor(loc, ..)
+            | Expression::AssignShiftLeft(loc, ..)
+            | Expression::AssignShiftRight(loc, ..)
+            | Expression::AssignAdd(loc, ..)
+            | Expression::AssignSubtract(loc, ..)
+            | Expression::AssignMultiply(loc, ..)
+            | Expression::AssignDivide(loc, ..)
+            | Expression::AssignModulo(loc, ..)
             | Expression::BoolLiteral(loc, _)
             | Expression::NumberLiteral(loc, _)
             | Expression::RationalNumberLiteral(loc, _)
@@ -383,7 +415,7 @@ impl Expression {
             | Expression::ArrayLiteral(loc, _)
             | Expression::List(loc, _)
             | Expression::Type(loc, _)
-            | Expression::Unit(loc, _, _)
+            | Expression::Unit(loc, ..)
             | Expression::This(loc)
             | Expression::Variable(Identifier { loc, .. })
             | Expression::AddressLiteral(loc, _) => *loc,
@@ -514,7 +546,7 @@ pub enum Statement {
     },
     Assembly {
         loc: Loc,
-        assembly: Vec<AssemblyStatement>,
+        assembly: AssemblyBlock,
     },
     Args(Loc, Vec<NamedArgument>),
     If(Loc, Expression, Box<Statement>, Option<Box<Statement>>),
@@ -537,17 +569,39 @@ pub enum Statement {
         Loc,
         Expression,
         Option<(Vec<(Loc, Option<Parameter>)>, Box<Statement>)>,
-        Option<Box<(Identifier, Parameter, Statement)>>,
-        Box<(Option<Parameter>, Statement)>,
+        Vec<CatchClause>,
     ),
     DocComment(Loc, CommentType, String),
 }
 
+#[derive(Debug, PartialEq, Clone)]
+pub enum CatchClause {
+    Simple(Loc, Option<Parameter>, Statement),
+    Named(Loc, Identifier, Parameter, Statement),
+}
+
 #[derive(Debug, PartialEq, Clone)]
 pub enum AssemblyStatement {
     Assign(Loc, AssemblyExpression, AssemblyExpression),
     LetAssign(Loc, AssemblyExpression, AssemblyExpression),
     Expression(AssemblyExpression),
+    If(Loc, AssemblyExpression, Box<AssemblyBlock>),
+    For(
+        Loc,
+        AssemblyBlock,
+        AssemblyExpression,
+        AssemblyBlock,
+        Box<AssemblyBlock>,
+    ),
+    Switch(
+        Loc,
+        AssemblyExpression,
+        Vec<AssemblySwitch>,
+        Option<AssemblySwitch>,
+    ),
+    Leave(Loc),
+    Break(Loc),
+    Continue(Loc),
 }
 
 #[derive(Debug, PartialEq, Clone)]
@@ -564,6 +618,14 @@ pub enum AssemblyExpression {
     Subscript(Loc, Box<AssemblyExpression>, Box<AssemblyExpression>),
 }
 
+pub type AssemblyBlock = Vec<AssemblyStatement>;
+
+#[derive(Debug, PartialEq, Clone)]
+pub enum AssemblySwitch {
+    Case(AssemblyExpression, Box<AssemblyBlock>),
+    Default(Box<AssemblyBlock>),
+}
+
 impl Statement {
     pub fn loc(&self) -> Loc {
         match self {
